@@ -47,7 +47,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <syslog.h>
-
+#include <sys/time.h>
+#include <sys/resource.h>
 
 #include "3ddesk.h"
 
@@ -88,8 +89,6 @@ int do_vpswitch = 1;
 
 
 
-//int goto_face = NO_GOTO;
-
 // Our display mode settings.
 int Light_On = 1;
 int Blend_On = 0;
@@ -113,9 +112,18 @@ float last_keypress = 0;
 // time of next "ghost keypress" when having fun
 float next_action = 0;
 
+int goto_on_flag = 0;
 
 int initiate_destroy = 0;   // starts the z offset zoom in
 
+Bool disable_autoacquire = false;
+
+Bool disable_random_fun = false;
+
+desktop_coords_t  last_coords;
+
+time_t time_of_last_acquire = 0;
+
 VDesktops vdesks;
 
 MessageManager msgmgr;
@@ -165,21 +173,26 @@ EventManager em;
 Bool end_X_event_loop;
 
 
-enum state_t {
-    STATE_ENTRY,
-    STATE_EXIT,
-    STATE_NORMAL,
-    STATE_GOTO,
-    STATE_SUSPENDED
-};
+#define STATE_ENTRY      0x00000001
+#define STATE_EXIT       0x00000002
+#define STATE_NORMAL     0x00000004
+#define STATE_GOTO       0x00000008
+#define STATE_SUSPENDED  0x00000010
 
+#define SET_STATE(s)     (state |=  (s))
+#define ISSET_STATE(s)   (state &   (s))
+#define UNSET_STATE(s)   (state &= ~(s))
+#define CLEAR_STATE()    (state =    0)
+
+unsigned int state = STATE_ENTRY;
 
-state_t state = STATE_ENTRY;
 
 // prototypes 
 
 static void goto_face_complete (Event *e);
-static void goto_face_begin (Event *e);
+static void switch_desktop_on_exit (Event *e);
+static void entry_move_start_hook (Event *e);
+static void exit_move_start_hook (Event *e);
 static void setup_goto_face(desktop_coords_t  goto_coords, 
                             desktop_coords_t  current_coord, 
                             desktop_coords_t  current_count);
@@ -206,23 +219,48 @@ static void load_digits(void);
 static void 
 goto_face_complete (Event *e)
 {
+    goto_on_flag = 0;
+
     //msgout (DEBUG, "-=-=-=-=- goto_face_complete\n");
-    begin_exit();
+    if (!cfg->options->disable_exit_after_goto)
+        begin_exit();
+    else 
+        cfg->options->disable_exit_after_goto = false;  // dont want this option "sticky"
+
+}
+
+// runs when a goto face reaches its destination (when setup by
+// setup_goto_face) and does not start an exit.
+static void 
+goto_face_complete2 (Event *e)
+{
+    goto_on_flag = 0;
 }
 
 
-// runs when entry movement stops
+// start the exit movement as soon as the entry movement is over when
+// where doing a "goto".
 static void 
-goto_face_begin (Event *e)
+goto_face_start_exit (Event *e)
 {
-    desktop_coords_t *desktop_to_goto = (desktop_coords_t *)(e->data);
+    //msgout (DEBUG, "-=-=-=-=- goto_face_start_exit\n");
+    if (!cfg->options->disable_exit_after_goto)
+        begin_exit();
+    else 
+        cfg->options->disable_exit_after_goto = false;  // dont want this option "sticky"
 
-    state = STATE_GOTO;
-    faces->goto_face (*desktop_to_goto);
+}
 
-    em.add_event (GOTO_FACE_COMPLETE, goto_face_complete);
 
-    delete desktop_to_goto;
+// runs when exit movement stops if do_vpswitch_early is false
+static void 
+switch_desktop_on_exit (Event *e)
+{
+    msgout (DEBUG, "Doing exit VD switch  c=%d   r=%d\n",
+            faces->get_current_column(),
+            faces->get_current_row() );
+    vdesks.set_vdesktop (faces->get_current_column(),
+                         faces->get_current_row() );
 }
 
 // runs when entry movement starts
@@ -253,8 +291,6 @@ setup_goto_face(desktop_coords_t  goto_coords,
                 desktop_coords_t  current_coord, 
                 desktop_coords_t  current_count)
 {
-    //int face = goto_face - 1;   // actual faces are 0 thru face_count - 1
-
     msgout (DEBUG, "**** setting up goto = %d x %d\n", goto_coords.column, goto_coords.row);
 
     switch (goto_coords.column) 
@@ -301,21 +337,21 @@ setup_goto_face(desktop_coords_t  goto_coords,
         goto_coords.row--;
     }
 
+    goto_on_flag = 1;
 
     msgout (DEBUG, "**** complete - going with = %d x %d\n", goto_coords.column, goto_coords.row);
 
+    desktop_coords_t dc(goto_coords);
 
-    desktop_coords_t *dc = new desktop_coords_t (goto_coords);
-    if (!dc) {
-        msgout(ERROR, "out of memory\n");
-        end_program(-1);
-    }
+    SET_STATE(STATE_GOTO);
+    faces->goto_face (dc);
 
-    em.add_event (ENTRY_MOVEMENT_STOP, 
-                  goto_face_begin, dc );
-    
+    em.add_event (GOTO_FACE_COMPLETE, goto_face_complete2);
+
+    em.add_event (ENTRY_MOVEMENT_STOP, goto_face_start_exit);
+
+} // END setup_goto_face
 
-} // END setup_goto_faces
 
 typedef struct portion_control_s {
     int inited;
@@ -414,7 +450,7 @@ uint digit_texture;
 
 
 static void draw_digits (int r, int c);
-static void draw_digit (int d);
+static void draw_desktop_number(int num);
 static void draw_digit (int d, float x_trans, float y_trans);
 static void draw_x (float x_trans, float y_trans);
 static void set_color (int color, float level);
@@ -455,21 +491,51 @@ set_color (int color, float level)
 }
 
 
+static void 
+draw_desktop_number(int num)
+{
+    float digit_size = cfg->options->digit_size;
+  
+    // where to draw to 
+    float centre_x = GLWin.get_width() / 2 ;
+    float centre_y = GLWin.get_height() - (digit_size / 2) ;
+  
+    // decode to digits
+ 
+    int numdigits = (int) log10((float)num);
+ 
+    float current_x = centre_x - (digit_size * numdigits / 2);
+  
+    while(num >= 10) {
+	draw_digit(num / 10, 
+		   current_x, 
+		   centre_y);
+	current_x += digit_size;
+  	num = num % 10; 
+    }
+
+    draw_digit(num, 
+               current_x, 
+               centre_y);
+}
+
+
+
 static void
 draw_digits (int r, int c)
 {
-    //draw_digit(r * c);
+    float digit_size = cfg->options->digit_size;
 
     draw_x (GLWin.get_width() / 2,
-            GLWin.get_height() - (cfg->options->digit_size / 2) );
+            GLWin.get_height() - (digit_size / 2) );
 
     draw_digit (r, 
-                GLWin.get_width() / 2 - (cfg->options->digit_size),
-                GLWin.get_height() - (cfg->options->digit_size / 2) );
+                GLWin.get_width() / 2 - (digit_size),
+                GLWin.get_height() - (digit_size / 2) );
 
     draw_digit (c, 
-                GLWin.get_width() / 2 + (cfg->options->digit_size),
-                GLWin.get_height() - (cfg->options->digit_size / 2) );
+                GLWin.get_width() / 2 + (digit_size),
+                GLWin.get_height() - (digit_size / 2) );
 
 }
 
@@ -519,35 +585,32 @@ draw_x (float x_trans, float y_trans)
     glPopMatrix();
 }
 
-static void 
-draw_digit (int d)
-{
-    // draw a digit in the middle
-    draw_digit (d, 
-                GLWin.get_width() / 2, 
-                GLWin.get_height() - (cfg->options->digit_size / 2) );
-}
-
 
 // this is where the digit is located in the texture
 float digit_texture_coords[] = 
-{ 0, 0, .25, 0, .5, 0, .75, 0, 0, .25, .25, .25, .5, .25, .75, .25, 
-  0, .5, .25, .5 };
+{ .25	, .5, 
+  0	, 0, 
+ .25	, 0, 
+ .5	, 0, 
+ .75	, 0, 
+ 0	, .25, 
+ .25	, .25, 
+ .5	, .25, 
+ .75	, .25, 
+ 0	, .5, 
+ .25	, .5 };
 
 static void
 draw_digit (int d, float x_trans, float y_trans)
 {
     float x1, y1, x2, y2;
 
-    if (d >= 10) {
-        msgout (DEBUG, "^^^^^^^^ above 10\n");
-        x1 = 0.25;  // show zero
-        y1 = 0.5;
-    } else {
-        x1 = digit_texture_coords[(d-1) * 2];
-        y1 = digit_texture_coords[(d-1) * 2 + 1];
-    }
-    
+    if (d >= 10)
+        return ;
+
+    x1 = digit_texture_coords[d * 2];
+    y1 = digit_texture_coords[d * 2 + 1];
+   
     x2 = x1 + 0.25;
     y2 = y1 + 0.25;
 
@@ -666,7 +729,7 @@ render_scene(void)
     // 
     if (current_time - last_time > cfg->options->animation_speed) {
 
-        if (state == STATE_EXIT) {
+        if (ISSET_STATE(STATE_EXIT)) {
 
             //if (!cfg->options->entry_exit_movement || faces->is_exit_movement_done()) {
             
@@ -675,18 +738,22 @@ render_scene(void)
             if (faces->is_exit_movement_done()) {
                 // quit (the z offset was changed to match the screen)
 
-                end_X_event_loop = True;
-		em.trigger_event (EXIT_MOVEMENT_STOP);
-                return;
+                if (!ISSET_STATE(STATE_GOTO)) {  // dont exit if we're still goto'ing
+                    UNSET_STATE(STATE_EXIT);
+                    end_X_event_loop = True;
+                    em.trigger_event (EXIT_MOVEMENT_STOP);
+                    return;
+                }
                 
             } else {
-	        faces->entry_exit_move();
+                faces->entry_exit_move();
             }
 
-        } else if (state == STATE_ENTRY) {
+        } else if (ISSET_STATE(STATE_ENTRY)) {
 
             if ( faces->is_entry_movement_done() ) {
-                state = STATE_NORMAL;
+                UNSET_STATE(STATE_ENTRY);
+                SET_STATE(STATE_NORMAL);
                 em.trigger_event (ENTRY_MOVEMENT_STOP);
 
             } else {
@@ -694,12 +761,13 @@ render_scene(void)
 
             }
 
-	} 
+        } 
 
         // change state to normal from goto if done goto
-        if (state == STATE_GOTO) {
+        if (ISSET_STATE(STATE_GOTO)) {
             if ( !faces->in_goto() ) {
-                state = STATE_NORMAL;
+                UNSET_STATE(STATE_GOTO);
+                SET_STATE(STATE_NORMAL);
                 em.trigger_event (GOTO_FACE_COMPLETE);
             }
         }
@@ -729,7 +797,8 @@ render_scene(void)
     // FIXME: use event to accomplish this?
 
     // after some seconds start some fun ;)
-    if (cfg->options->random_fun_delay &&
+    if (!disable_random_fun && 
+        cfg->options->random_fun_delay &&
         current_time - last_keypress > cfg->options->random_fun_delay * 1000) 
     {
         if ((unsigned long)current_time >= (unsigned long)next_action) {
@@ -869,7 +938,7 @@ render_scene(void)
             else
                 n = faces->get_current_row() + 1;
 
-            draw_digit (n);
+            draw_desktop_number (n);
         }
 
 #if 0  // use a string rather then the digit
@@ -928,20 +997,66 @@ mouse_pressed(unsigned int button)
 
     switch (button) {
     case 1: // left
-    case 5: // mouse wheel down
-        faces->goto_left();
+    	if (cfg->options->alt_mousebuttons)
+            begin_exit();
+        else
+            if (cfg->options->swap_mousebuttons)
+                faces->goto_right();
+            else
+                faces->goto_left();
         break;
 
     case 3: // right
-    case 4: // mouse wheel up
-        faces->goto_right();
+        if (cfg->options->alt_mousebuttons) {
+            SET_STATE(STATE_GOTO);
+            faces->goto_face(last_coords);
+            begin_exit();
+        } else 
+            if (cfg->options->swap_mousebuttons)
+                faces->goto_left();
+            else
+                faces->goto_right();
         break;
 
     case 2: // middle
-
-        begin_exit();
-
+        if (cfg->options->alt_mousebuttons) {
+            SET_STATE(STATE_GOTO);
+            faces->goto_face(last_coords);
+        } else {
+            begin_exit();
+        }
         break;
+
+
+    case 4: // mouse wheel up
+        if (cfg->options->reverse_mousewheel)
+            faces->goto_left();
+        else
+            faces->goto_right();
+        break;
+
+    case 5: // mouse wheel down
+        if (cfg->options->reverse_mousewheel)
+            faces->goto_right();
+        else
+            faces->goto_left();
+        break;
+
+
+    case 6: // extra button on some mice, do we use this?
+        if (cfg->options->swap_mousebuttons)
+            faces->goto_left();
+        else
+            faces->goto_right();
+        break;
+
+    case 7: // extra button on some mice, do we use this?
+        if (cfg->options->swap_mousebuttons)
+            faces->goto_right();
+        else
+            faces->goto_left();
+        break;
+
     }
 } // END mouse_pressed
 
@@ -954,6 +1069,11 @@ key_pressed(KeySym key)
 {
     static KeySym last_key_pressed = 0;
 
+    if (cfg->disable_keys_in_goto &&
+        goto_on_flag && 
+        !cfg->options->disable_exit_after_goto)
+        return;
+
     last_keypress = get_milli_time();
 
     next_action = 0;
@@ -967,8 +1087,12 @@ key_pressed(KeySym key)
             glEnable(GL_BLEND);
         break;
 
-    case 108: case 76:  // L - Lighting
-        Light_On = Light_On ? 0 : 1; 
+    case 108: case 76:  // L - last desktop
+        SET_STATE(STATE_GOTO);
+        faces->goto_face(last_coords);
+        
+        // this used to be for turning lights on an off...
+        //Light_On = Light_On ? 0 : 1; 
         break;
 
     case 109: case 77:  // M - Mode of Blending
@@ -991,6 +1115,10 @@ key_pressed(KeySym key)
         faces->goto_random_face();
         break;
 
+    case 102: case 70: // F - Fun Mode
+        disable_random_fun = disable_random_fun ? 0 : 1;
+        break;
+
 #if 1 // for debugging best light position...
     case 119: // w
         light_position[0] -= 0.5;
@@ -1026,7 +1154,7 @@ key_pressed(KeySym key)
         }
         last_key_pressed = key;
 
-        state = STATE_GOTO;
+        SET_STATE(STATE_GOTO);
         faces->goto_face(9);
         break;
         
@@ -1050,14 +1178,14 @@ key_pressed(KeySym key)
         if (initiate_destroy) {
 
             initiate_destroy = 0;
-	    faces->exit_movement_cancel();
+            faces->exit_movement_cancel();
 
             // FIXME: replace with event something or other
-            state = STATE_GOTO;
+            SET_STATE(STATE_GOTO);
             faces->goto_face(key - 49);
 
         } else {
-            state = STATE_GOTO;
+            SET_STATE(STATE_GOTO);
             faces->goto_face(key - 49);
         }
 
@@ -1117,16 +1245,16 @@ key_pressed(KeySym key)
 Arrangement *arrangement_array[FACE_TYPE_LAST];
 
 // assumes 'a' is Arrangement *a;
-#define  ALLOC_ARRANGEMENT(enum_name, type) 			\
-    if (arrangement_array[enum_name]) 				\
-        delete (arrangement_array[enum_name]);			\
-    a = new type (vdesks,					\
-                  face_width, face_height,			\
-                  face_set, cfg);				\
-    if (!a) {							\
-        msgout(ERROR, "out of memory\n");			\
-        end_program (1);					\
-    }								\
+#define  ALLOC_ARRANGEMENT(enum_name, type)                     \
+    if (arrangement_array[enum_name])                           \
+        delete (arrangement_array[enum_name]);                  \
+    a = new type (vdesks,                                       \
+                  face_width, face_height,                      \
+                  face_set, cfg);                               \
+    if (!a) {                                                   \
+        msgout(ERROR, "out of memory\n");                       \
+        end_program (1);                                        \
+    }                                                           \
     arrangement_array[enum_name] = a;
 
 
@@ -1186,6 +1314,7 @@ demand_load_arrangement (FaceType_t type)
         case FACE_TYPE_CAROUSEL:
             f = new CarouselArrangement (vdesks, face_width, face_height, face_set, cfg);
             break;
+#if !defined (ONE_ARRANGEMENT_ONLY)
         case FACE_TYPE_CYLINDER:
             f = new CylinderArrangement (vdesks, face_width, face_height, face_set, cfg);
             break;
@@ -1201,6 +1330,7 @@ demand_load_arrangement (FaceType_t type)
         case FACE_TYPE_FLIP:
             f = new FlipArrangement (vdesks, face_width, face_height, face_set, cfg);
             break;
+#endif
         default:
             f = NULL;
         }
@@ -1263,21 +1393,26 @@ begin_exit (void)
         // if no "zooming" then we don't want to exit until the goto
         // face is complete
         msgout (DEBUG, "cant exit in goto....\n");
-        state = STATE_GOTO;
+        SET_STATE(STATE_GOTO);
         em.add_event (GOTO_FACE_COMPLETE, goto_face_complete);
         return;
     }
 
-    state = STATE_EXIT;
+    SET_STATE(STATE_EXIT);
     em.trigger_event (EXIT_MOVEMENT_START);
     initiate_destroy = 1;
 
     if (do_vpswitch) {
-        msgout (DEBUG, "Doing VD switch  c=%d   r=%d\n",
-                faces->get_current_column(),
-                faces->get_current_row() );
-        vdesks.set_vdesktop (faces->get_current_column(),
-                             faces->get_current_row() );
+
+        if (cfg->early_desktop_switch) {
+            msgout (DEBUG, "Doing VD switch  c=%d   r=%d\n",
+                    faces->get_current_column(),
+                    faces->get_current_row() );
+            vdesks.set_vdesktop (faces->get_current_column(),
+                                 faces->get_current_row() );
+        } else {
+            em.add_event (EXIT_MOVEMENT_STOP, switch_desktop_on_exit);
+        }
     }
 
     // the z_offset translation starts and is carried out by the
@@ -1308,7 +1443,12 @@ wait_for_activation(void)
     goto_coords.column = NO_GOTO;
     goto_coords.column = NO_GOTO;
 
-    state = STATE_SUSPENDED;
+    CLEAR_STATE();
+    SET_STATE(STATE_SUSPENDED);
+
+    disable_autoacquire = false;
+    if (cfg->autoacquire)
+        alarm(cfg->autoacquire);
 
     do {
 
@@ -1340,6 +1480,11 @@ wait_for_activation(void)
 
         } while (errno == ENOMSG);
 #else
+        if (msgmgr.flush() < 0) {
+            msgout (ERROR, "flush failed: %s\n", strerror(errno));
+            end_program(-1);
+        }
+
         // should block until message received
         if (msgmgr.recieve() < 0) {
             msgout (ERROR, "msgrcv failed: %s\n", strerror(errno));
@@ -1351,6 +1496,8 @@ wait_for_activation(void)
 
         msgout (DEBUG, "MSG: %ld\n", msgmgr.msg.mtype);
 
+        disable_autoacquire = true;
+
         // must get this now in case the current desktop was changed by
         // other means
         vdesks.get_vdesktop_info (&current_col, &current_row, &col_count, &row_count);
@@ -1415,12 +1562,13 @@ wait_for_activation(void)
             continue;  // don't activate
 
             break;
+
         case MTYPE_ACTIVATE:
 
             if (strlen(msgmgr.msg.view)) {
                 // client specified a view - ignore cmd line params except goto
 
-                cfg->reload(); // always reload cfg is view specified
+                cfg->reload(); // always reload cfg if view specified
 
                 cfg->set_config_set (msgmgr.msg.view);
 
@@ -1429,8 +1577,6 @@ wait_for_activation(void)
 
             } else {
 
-                // FIXME: track .conf timestamp for reloading efficiency?
-
                 cfg->reload(); // reload in case "default" changed
                 
                 cfg->set_config_set ("default");
@@ -1458,6 +1604,18 @@ wait_for_activation(void)
 
             }
 
+            if (msgmgr.msg.disable_exit_after_goto)
+                cfg->options->disable_exit_after_goto = true;
+
+            if (msgmgr.msg.disable_random_fun)
+                disable_random_fun = 0;
+
+            if (msgmgr.msg.reverse_mousewheel)
+                cfg->options->reverse_mousewheel = true;
+            if (msgmgr.msg.swap_mousebuttons)
+                cfg->options->swap_mousebuttons = true;
+            if (msgmgr.msg.alt_mousebuttons)
+                cfg->options->alt_mousebuttons = true;
 
             // if we are asked to goto the current row and column then
             // do nothing
@@ -1524,7 +1682,7 @@ wait_for_activation(void)
         }
     } while (!done);
 
-    state = STATE_NORMAL;
+    SET_STATE(STATE_NORMAL);
 
     msgout (DEBUG, "Activated\n");
     
@@ -1577,6 +1735,37 @@ idle_func(void)
 } // END idle_func
 #endif
 
+void 
+acquire_desktop (int rv)
+{
+    if (disable_autoacquire || !cfg->autoacquire)
+        return;
+
+    int current_row = 0, current_col = 0;
+    int row_count = 0, col_count = 0;
+
+    vdesks.get_vdesktop_info (&current_col, &current_row, &col_count, &row_count);
+
+    msgout (DEBUG, ")))) GRABBING %d x %d\n", current_col, current_row);
+
+    if (GLWin.grab_screenshot_data() < 0) {
+        msgout (ERROR, "couldn't grab screen image\n");
+        end_program(-1);
+    }
+
+    time_of_last_acquire = time(NULL);
+    
+    face_set->load_texture_data(current_col + (current_row * col_count), 
+                                cfg->texture_size,
+                                GLWin.get_screenshot_data_ptr() );
+
+    faces->make_display_list();
+
+    if (cfg->autoacquire)
+        alarm(cfg->autoacquire);
+
+} // END acquire_desktop
+
 
 static int 
 main_loop(void)
@@ -1585,12 +1774,6 @@ main_loop(void)
     KeySym key;
     Bool quit = False;
 
-
-    Atom blackbox_notify_current_workspace =
-        XInternAtom(GLWin.get_display(), "_BLACKBOX_NOTIFY_CURRENT_WORKSPACE", False);
-    Atom blackbox_notify_workspace_count =
-        XInternAtom(GLWin.get_display(), "_BLACKBOX_NOTIFY_WORKSPACE_COUNT", False);
-
     desktop_coords_t  goto_coords;
     desktop_coords_t  current_count;
     desktop_coords_t  current_coords;
@@ -1599,6 +1782,7 @@ main_loop(void)
 
         goto_coords.column = NO_GOTO;
         goto_coords.column = NO_GOTO;
+        goto_on_flag = 0;
 
         goto_coords = wait_for_activation();
 
@@ -1607,10 +1791,13 @@ main_loop(void)
         current_count.column = faces->get_number_of_columns();
         current_count.row = faces->get_number_of_rows();
         
+        // last coords are used when you press L or in alt_mousebuttons
+        last_coords = current_coords;
 
 #ifndef SCAN_MODE
-        if (do_screenshot) {
-        
+        if (do_screenshot &&
+            (time(NULL) - time_of_last_acquire) > 1) {
+            
             if (GLWin.grab_screenshot_data() < 0) {
                 msgout (ERROR, "couldn't grab screen image\n");
                 end_program(-1);
@@ -1621,10 +1808,14 @@ main_loop(void)
                                         GLWin.get_screenshot_data_ptr() );
 
             faces->make_display_list();
-        }
+        } 
+        //else {
+        //    msgout(DEBUG, "~~~~~~~ NOT GRABBING\n");
+        //}
 #endif
 
-        GLWin.unhide_GL_window();
+        if (cfg->options->do_fullscreen)
+            GLWin.unhide_GL_window();
 
         initiate_destroy = 0;
 
@@ -1636,8 +1827,8 @@ main_loop(void)
                             current_count);
         }
 
-	state = STATE_ENTRY;
-	em.trigger_event (ENTRY_MOVEMENT_START);
+        SET_STATE(STATE_ENTRY);
+        em.trigger_event (ENTRY_MOVEMENT_START);
 
 
         end_X_event_loop = False;
@@ -1685,20 +1876,6 @@ main_loop(void)
                         quit = True;
                     }
 
-                    if ((unsigned)event.xclient.data.l[0]==
-                        blackbox_notify_current_workspace)
-                    {
-                        msgout (DEBUG, "WOO HAA current ws is %d\n", (int)event.xclient.data.l[1]);
-                    }
-                    else if ((unsigned)event.xclient.data.l[0]==
-                             blackbox_notify_workspace_count)
-                    {
-                        msgout (DEBUG, "WOO HAA !double! ws count is %d\n", event.xclient.data.l[1]);
-                    } else {
-                        msgout (DEBUG, ">>>>> some other ClientMessage\n");
-                    }
-
-                    
                     break;
                 default:
                     break;
@@ -1712,7 +1889,8 @@ main_loop(void)
 
         }
 
-        GLWin.hide_GL_window();
+        if (cfg->options->do_fullscreen)
+            GLWin.hide_GL_window();
 
     }
 
@@ -1839,6 +2017,7 @@ load_digits(void)
 } // END load_digits
 
 
+
 // accesses global cfg
 static void
 initialize_3ddesktop (void)
@@ -1918,6 +2097,12 @@ initialize_3ddesktop (void)
     sigaction (SIGINT, &sa, NULL);
     sigaction (SIGQUIT, &sa, NULL);
 
+    sa.sa_mask = mask;
+    sa.sa_handler = acquire_desktop;
+    sigaction (SIGALRM, &sa, NULL);
+
+    if (cfg->autoacquire)
+        alarm(cfg->autoacquire);
 
     // open the messaging mechanism (message queue) we will listen on
     
@@ -1926,12 +2111,6 @@ initialize_3ddesktop (void)
         end_program(-1);
     }
 
-    if (flag_that_i_am_running()) {
-        // (if we've daemonized this goes to syslog which is not easily seen)
-        msgout(ERROR, "Oops, someone beat us to start up!!  Amazing!\n");
-        end_program(-1);
-    }
-
     if (cfg->make_pidfile() < 0)
         end_program(-1);
 
@@ -1952,7 +2131,8 @@ initialize_3ddesktop (void)
     // Loads up the correct perspective matrix
     GLWin.resize_scene();
 
-    GLWin.hide_GL_window();
+    if (cfg->options->do_fullscreen)
+        GLWin.hide_GL_window();
 
     // this must be here after opengl is initialized since the FaceSet
     // constructor loads the digit textures.
@@ -1980,8 +2160,10 @@ initialize_3ddesktop (void)
 
                 vdesks.set_vdesktop (j, i);
 
-                if (cfg->options->acquire > 1) 
+                if (cfg->options->acquire > 1) {
+                    msgout(DEBUG, "sleeping %d...\n", cfg->options->acquire);
                     sleep_ms (cfg->options->acquire);
+                }
                 
                 if (GLWin.grab_screenshot_data() < 0) {
                     msgout (ERROR, "couldn't grab screen image\n");
@@ -2001,6 +2183,12 @@ initialize_3ddesktop (void)
 
     load_digits();
 
+    if (flag_that_i_am_running()) {
+        // (if we've daemonized this goes to syslog which is not easily seen)
+        msgout(ERROR, "Oops, someone beat us to start up!!  Amazing!\n");
+        end_program(-1);
+    }
+
     em.add_event (ENTRY_MOVEMENT_START, entry_move_start_hook, NULL, 1);
     em.add_event (EXIT_MOVEMENT_START, exit_move_start_hook, NULL, 1);
                       
@@ -2022,6 +2210,11 @@ int main(int argc,char **argv)
 
     initialize_3ddesktop();
 
+    int rc = setpriority(PRIO_PROCESS, 0, cfg->priority);
+    if (rc < 0) {
+        msgout(DEBUG, "~~~~~ failed to set priority %d\n", cfg->priority);
+    }
+
     return main_loop();
 
 }  // END main
