@@ -68,6 +68,7 @@ void Arrangement::set_current_face (int col, int row)
 {
     current_col = col;
     current_row = row;
+    current_face = col + (row * n_columns);
 }
 
 int Arrangement::get_current_column(void) 
@@ -229,6 +230,7 @@ float Arrangement::goto_face (desktop_coords_t dest)
 
     current_col = dest.column; //newface; 
     current_row = dest.row;
+    current_face = newface;
     
     return cfg->options->face_change_steps;
 }
@@ -657,6 +659,8 @@ CylinderArrangement::CylinderArrangement(VDesktops &vdesks,
 					 Config *c)
     : Arrangement (fs, c)
 {
+    two_face_gap = 3;
+
     z_offset_distance = 5.0;
 
     X_Rot = 0.0;
@@ -1386,6 +1390,7 @@ void LinearArrangement::set_current_face (int c, int r)
 
     current_row = r;
     current_col = c;
+    current_face = c + (r * n_columns);
 
     X_Trans = -(half_face_width + (current_col * (face_width + between)));
     Y_Trans = ((current_row * (face_height + between)) + half_face_height);
@@ -1665,13 +1670,13 @@ FlipArrangement::FlipArrangement(VDesktops & vdesks,
     face_width = fw;
     face_height = fh;
 
-    face_A = new Face (1);
+    face_A = new Face (0);
     if (!face_A) {
         msgout(ERROR, "out of memory\n");
         exit (-1);
     }
 
-    face_B = new Face (1);
+    face_B = new Face (0);
     if (!face_B) {
         msgout(ERROR, "out of memory\n");
         exit (-1);
