@@ -38,7 +38,7 @@
 #include <getopt.h>
 
 #include "3ddesk.h"
-//#include "arrange.hpp"
+#include "util.h"
 
 // STL
 using namespace std;
@@ -55,12 +55,10 @@ usage_and_bye (void)
            "                       (sleep for # millisecs at each screen for refresh)\n"
            "    --dir=/path      the install dir if different than /usr/share/3ddesktop\n"
            "    --texturesize=#  size of textures (larger for more detail) in powers of 2\n" 
-           "    --kde2           operate with KDE 2\n"
-           "    --kde3           operate with KDE 3\n"
-           "    --gnome2         operate with GNOME 2\n"
-           "    --ewmh           Use EWMH standard for desktops (GNOME 2, KDE 3 and others)\n"
-           "    --workspaces     switch between workspace desktops instead of grid areas\n"
-           "    --sawfish        use sawfish-client to switch workspaces\n"
+           "    --wm=            Specify the Windowmanager type so all the virtual \n"
+           "                     desktops are found correctly.  Available options are:\n"
+           "                       kde2, kde3, gnome1, gnome2, ewmh, fluxbox, windowmaker,\n"
+           "                       enlightenment, sawfishonly, workspaces\n"
            "    --fastest        hog the CPU for smoothest operation\n"
            "    -v               verbose/debug information\n\n");
     exit (1);
@@ -74,7 +72,6 @@ public:
     char name[50];
 
     // bits/bools on/off
-    int acquire : 1;
     int show_fps : 1;
     int show_digits : 1;
     int do_fullscreen : 1;
@@ -82,6 +79,11 @@ public:
     int use_context_switch : 1;
     int use_breathing : 1;
     int daemonized : 1;
+    int use_wireframe : 1;
+    int glcompression : 1;
+
+    int __early_desktop_switch : 1;
+    int __disable_keys_in_goto : 1;
 
     int __verbose : 1;
     int __texture_size;
@@ -90,6 +92,13 @@ public:
     int __use_ewmh : 1;
     int __sawfish_only : 1;
     int __use_workspaces : 1;
+    int __use_viewareas : 1;
+
+    int __autoacquire;
+
+    int __priority;
+
+    int acquire;
 
 #if defined(__FreeBSD__)
     char base_dir[MAXNAMLEN + 1];
@@ -122,6 +131,11 @@ public:
 
     int recalc;  // set if arrangement needs to be recalculated
 
+    int disable_exit_after_goto;
+
+    int reverse_mousewheel;
+    int swap_mousebuttons;
+    int alt_mousebuttons;
 
     // ---------
     //Arrangement *arrangement;
@@ -140,8 +154,14 @@ public:
 	__use_ewmh = 0;
         __sawfish_only = 0;
         __use_workspaces = 0;
+        __use_viewareas = 0;
+        __early_desktop_switch = 1;
+        __disable_keys_in_goto = 1;
+        __autoacquire = 2;
+        __priority = 12;
 
         acquire = 0;
+        glcompression = 0;
         show_fps = 0;
         show_digits = 1;
         do_fullscreen = 1;
@@ -161,6 +181,7 @@ public:
         digit_color = COLOR_RED;
 
         frame_color = COLOR_GRAY;
+        use_wireframe = false;
 
         linear_spacing = 2.0;
 
@@ -171,6 +192,12 @@ public:
         face_change_steps = 40;
         zoom_steps = 45;
 
+        disable_exit_after_goto = 0;
+
+        reverse_mousewheel = 0;
+        swap_mousebuttons = 0;
+        alt_mousebuttons = 0;
+
 	// ----
         //arrangement = NULL;
 
@@ -219,7 +246,7 @@ public:
         return 0;  // default false (?)
     }
 
-    int set_option (char *option, char *value) {
+    int set_option (char *option, char *value, bool from_cmdline) {
         int tmpint;
         float tmpfloat;
 
@@ -282,6 +309,9 @@ public:
 	} else if (strcmp(option, "frame_color") == 0) {
             set_color (&frame_color, value);
 
+	} else if (strcmp(option, "use_wireframe") == 0) {
+            use_wireframe = get_boolean (value);
+
 	} else if (strcmp(option, "fps") == 0) {
 	    show_fps = get_boolean (value);
 
@@ -297,7 +327,7 @@ public:
 		random_fun_delay = tmpint;
 
 	} else if (strcmp(option, "workspaces") == 0) {
-	    __use_workspaces = get_boolean (value);
+            __use_workspaces = from_cmdline ? 1 : get_boolean (value);
 
 	} else if (strcmp(option, "texturesize") == 0) {
 	    __texture_size = atoi (value);
@@ -325,25 +355,34 @@ public:
 		default_z_offset_distance = tmpfloat;
 
 	} else if (strcmp(option, "win") == 0) {
-	    do_fullscreen = 0;
+	    do_fullscreen = from_cmdline ? 0 : get_boolean (value);
 
 	} else if (strcmp(option, "sawfish") == 0) {
-	    __sawfish_only = 1;
+	    __sawfish_only = from_cmdline ? 1 : get_boolean (value);
 
         } else if (strcmp(option, "kde") == 0) {
-            __use_kde = 1;
+            __use_kde = from_cmdline ? 1 : get_boolean (value);
 
         } else if (strcmp(option, "kde2") == 0) {
-            __use_kde = 1;
+            __use_kde = from_cmdline ? 1 : get_boolean (value);
 
         } else if (strcmp(option, "kde3") == 0) {
-            __use_ewmh = 1;
+            __use_ewmh = from_cmdline ? 1 : get_boolean (value);
+
+        } else if (strcmp(option, "gnome") == 0) {
+            // "gnome off" will actaully enable in .conf ... but oh well
+            __use_ewmh = 0;
+            __use_workspaces = 1;
+
+        } else if (strcmp(option, "gnome1") == 0) {
+            __use_ewmh = 0;
+            __use_workspaces = 1;
 
         } else if (strcmp(option, "gnome2") == 0) {
-            __use_ewmh = 1;
+            __use_ewmh = from_cmdline ? 1 : get_boolean (value);
 
         } else if (strcmp(option, "ewmh") == 0) {
-            __use_ewmh = 1;
+            __use_ewmh = from_cmdline ? 1 : get_boolean (value);
 
 	} else if (strcmp(option, "acquire") == 0) {
 	    acquire = 1;
@@ -360,14 +399,29 @@ public:
 
 	    if (strcmp (value, "sawfishonly") == 0) {
 		__sawfish_only = 1;
-	    } else if (strcmp (value, "kde") == 0) {
+	    } else if (strcmp (value, "kde") == 0 ||
+                       strcmp (value, "kde2") == 0) {
 		__use_kde = 1;
-	    } else if (strcmp (value, "gnome2") == 0) {
+		__use_ewmh = 0;
+	    } else if (strcmp (value, "kde3") == 0) {
 		__use_ewmh = 1;
-	    } else if (strcmp (value, "ewmh") == 0) {
-		__use_ewmh = 1;
-	    } else if (strcmp (value, "gnomeworkspaces") == 0) {
+	    } else if (strcmp (value, "gnome") == 0 ||
+                       strcmp (value, "gnome1") == 0 ||
+                       strcmp (value, "enlightenment") == 0) {
+                __use_viewareas = 1;
 		__use_workspaces = 1;
+		__use_ewmh = 0;
+	    } else if (strcmp (value, "gnome2") == 0 ||
+                       strcmp (value, "ewmh") == 0) {
+		__use_ewmh = 1;
+	    } else if (strcmp (value, "gnomeworkspaces") == 0 ||
+                       strcmp (value, "windowmaker") == 0 ||
+                       strcmp (value, "workspaces") == 0) {
+		__use_workspaces = 1;
+		__use_ewmh = 0;
+	    } else if (strcmp (value, "fluxbox") == 0) {
+		__use_ewmh = 1;
+                __early_desktop_switch = 0;
 	    } else {
 		return 1;
 	    }
@@ -402,6 +456,33 @@ public:
         } else if (strcmp(option, "verbose") == 0) {
             __verbose = get_boolean (value);
 
+        } else if (strcmp(option, "early_desktop_switch") == 0) {
+            __early_desktop_switch = get_boolean (value);
+
+        } else if (strcmp(option, "disable_keys_in_goto") == 0) {
+            __disable_keys_in_goto = get_boolean (value);
+
+        } else if (strcmp(option, "autoacquire") == 0) {
+            __autoacquire = atoi (value);
+            
+        } else if (strcmp(option, "dontexit") == 0) {
+            disable_exit_after_goto = get_boolean (value);
+
+        } else if (strcmp(option, "glcompression") == 0) {
+            glcompression = get_boolean (value);
+
+        } else if (strcmp(option, "priority") == 0) {
+            __priority = atoi (value);
+
+        } else if (strcmp(option, "reverse_mousewheel") == 0) {
+            reverse_mousewheel = get_boolean (value);
+
+        } else if (strcmp(option, "swap_mousebuttons") == 0) {
+            swap_mousebuttons = get_boolean (value);
+
+        } else if (strcmp(option, "alt_mousebuttons") == 0) {
+            alt_mousebuttons = get_boolean (value);
+
 	}
 
 	return 0;  // success
@@ -428,6 +509,9 @@ public:
             {"gnome2", 0, 0, 0},
             {"ewmh", 0, 0, 0},
             {"kde2", 0, 0, 0},
+            {"kde3", 0, 0, 0},
+            {"gnome", 0, 0, 0},
+            {"gnome1", 0, 0, 0},
             {0, 0, 0, 0}
         };
 
@@ -443,7 +527,7 @@ public:
 	      
                 //msgout (DEBUG, "option %s", long_options[option_index].name);
                 if (set_option ((char *)long_options[option_index].name, 
-                                optarg))
+                                optarg, 1))
                     usage_and_bye();
                 break;
             case 'v':
@@ -487,14 +571,20 @@ public:
     char pidfile[PATH_MAX];
 #endif
 
-    bool use_kde;
-    bool use_ewmh;
-    bool sawfish_only;
-    bool use_workspaces;
+    int use_kde : 1;
+    int use_ewmh : 1;
+    int sawfish_only : 1;
+    int use_workspaces : 1;
+    int use_viewareas : 1;
+    int early_desktop_switch : 1;
+    int disable_keys_in_goto : 1;
+    int verbose : 1;
 
     int texture_size;
 
-    int verbose;
+    int autoacquire;
+
+    int priority;
 
     // ptr to the current option set (either cmd line or a conf "view")
     Options *options;
@@ -529,8 +619,12 @@ public:
         use_kde = cmd_line_options->__use_kde;
         use_ewmh = cmd_line_options->__use_ewmh;
         use_workspaces = cmd_line_options->__use_workspaces;
+        use_viewareas = cmd_line_options->__use_viewareas;
         sawfish_only = cmd_line_options->__sawfish_only;
-
+        early_desktop_switch = cmd_line_options->__early_desktop_switch;
+        disable_keys_in_goto = cmd_line_options->__disable_keys_in_goto;
+        autoacquire = cmd_line_options->__autoacquire;
+        priority = cmd_line_options->__priority;
     }
 
     void init (int argc, char **argv) {
@@ -548,17 +642,31 @@ public:
             return;
         }
 
+        int do_random = -1;
+        if (!strcmp(name, "random")) {
+            msgout (DEBUG, "*** picking random view\n");
+            do_random = get_randomi(0, config_options.size());
+        }
+
         list<Options *>::iterator k;
-        for (k = config_options.begin(); k != config_options.end(); k++) {
-            if ( !strcmp ((*k)->name, name) ) {
+        int cnt = 0;
+        for (k = config_options.begin(); k != config_options.end(); k++, cnt++) {
+
+            if ( (do_random != -1 && cnt == do_random) 
+                 ||
+                 (do_random == -1 && !strcmp ((*k)->name, name)) )
+            {
                 options = *k;
-                msgout (DEBUG, "*** FOUND cfg set %s\n", name);
+                msgout (DEBUG, "*** FOUND cfg set %s\n", (*k)->name);
                 break;
             }
         }
     }
 
     void reload () {
+
+        // FIXME: track .conf timestamp for reloading efficiency?
+
         list<Options *>::iterator k;
         for (k = config_options.begin(); k != config_options.end(); k++)
             delete *k;
@@ -568,6 +676,9 @@ public:
     }
 
     void load_conf () {
+
+        // FIXME: track .conf timestamp for reloading efficiency?
+
         // Try opening  ~/.3ddesktop/3ddesktop.conf
         //       then   /usr/share/3ddesktop/3ddesktop.conf
 
@@ -587,6 +698,8 @@ public:
                 return;
             }
         }
+        
+        msgout (DEBUG, "load_conf: opened %s\n", config_file);
 
         char var1[100];
         char var2[100];
@@ -650,13 +763,13 @@ public:
 		//msgout(DEBUG, "not in a option set\n");
 
                 // Treat these as "global" options 
-                cmd_line_options->set_option(var1, var2);
+                cmd_line_options->set_option(var1, var2, 0);
 
                 sync_global_options();
 
 	    } else {
 
-                o->set_option(var1, var2);
+                o->set_option(var1, var2, 0);
 
             }
 
